﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;



namespace FixPoseChannels
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void SelectFigure_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                Figure.Text = dlg.FileName;
            }
        }

        private void SelectPose_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                Pose.Text = dlg.FileName;
            }
        }

        private void Go_Click(object sender, EventArgs e)
        {
            PoserFigure figure = new PoserFigure();

            // First, read in the figure and build a list of channels
            using (StreamReader reader = File.OpenText(Figure.Text))
            {
                FigureParseMachine.Start();

                string CurrentLine = reader.ReadLine();
                while (CurrentLine != null)
                {
                    Status.Text = "Reading CR2 File.";
                    Application.DoEvents();

                    if (CurrentLine.Contains("readScript"))
                    {
                        string scriptFileName = CurrentLine.Replace("readScript ", "").Replace("\"", "").Trim();
                        Status.Text = "ReadScript '" + scriptFileName + "'";
                        Application.DoEvents();
                        ReadScriptResolver resolver = new ReadScriptResolver(runtimeFolder.Text, scriptFileName);
                        string subLine = resolver.readLine();
                        while (subLine != null)
                        {
                            PoserActor actor = FigureParseMachine.ProcessLine(subLine);
                            if (actor != null) figure.Actors.Add(actor);
                            subLine = resolver.readLine();
                        }
                    }
                    else
                    {
                        PoserActor actor = FigureParseMachine.ProcessLine(CurrentLine);
                        if (actor != null) figure.Actors.Add(actor);
                    }
                    CurrentLine = reader.ReadLine();
                }
            }

            PoserActor lastActor = FigureParseMachine.getLastActor();
            if (lastActor != null) figure.Actors.Add(lastActor);

            // Step one: Inject the names into the channels - otherwise you have to start with
            // a CR2 that was already injected, which rather defeats the purpose.

            using (StreamReader reader = File.OpenText(Pose.Text))
            {
                PoseParseMachine.StartForName();
                Status.Text = "Reading PZ2 file and modifying channel names.";
                Application.DoEvents();

                string CurrentLine = reader.ReadLine();
                while (CurrentLine != null)
                {
                    PoseFix fix = PoseParseMachine.ProcessLineForName(CurrentLine);
                    if (fix != null)
                    {
                        if(!String.IsNullOrEmpty(fix.Name)) figure.UpdateDisplayName(fix);
                    }
                    CurrentLine = reader.ReadLine();
                }
            }

            // Step two: build the new PZ2.

            using (StreamReader reader = File.OpenText(Pose.Text))
            {
                using (StreamWriter writer = File.CreateText(Pose.Text + ".fix"))
                {
                    Status.Text = "Resolving ERC link names and creating new PZ2 file.";
                    Application.DoEvents();
                    PoseParseMachine.Start();

                    string CurrentLine = reader.ReadLine();
                    while (CurrentLine != null)
                    {
                        PoseFix fix = PoseParseMachine.ProcessLine(CurrentLine);
                        if (fix != null)
                        {
                            string newChannel = figure.LookForAlternate(fix);
                            if (newChannel != null)
                            {
                                CurrentLine = CurrentLine.Replace(fix.Channel, newChannel);
                            }
                        }
                        writer.WriteLine(CurrentLine);
                        CurrentLine = reader.ReadLine();
                    }
                }
            }

            Status.Text = "Backing up old pose file";
            Application.DoEvents();
            File.Move(Pose.Text, Pose.Text + ".old");
            File.Move(Pose.Text + ".fix", Pose.Text);

            MessageBox.Show("All done!", "Channel Name Repair");

            Status.Text = "Ready...";
            Application.DoEvents();
        }
    }
}
